"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
// tslint:disable-next-line: file-name-casing
const cli_ux_1 = require("cli-ux");
const fs = require("fs");
const path = require("path");
const readline = require("readline");
const stream = require("stream");
const searchIssues_1 = require("../jira/searchIssues");
const teamUtils_1 = require("../misc/teamUtils");
const jiraUtils_1 = require("../misc/jiraUtils");
/*
    Fetches all initiatives
*/
const fetchInitiatives = async (userConfig, cacheDir, useCache) => {
    var e_1, _a;
    cli_ux_1.default.action.start('Fetching roadmap initiatives using: ' +
        userConfig.roadmap.jqlInitiatives +
        ' ');
    let issues = [];
    // If cache is enabled we don't fetch initiatives twice on the same day
    const today = new Date();
    const initiativesCache = path.join(cacheDir, 'roadmap-initiatives-' + today.toJSON().slice(0, 10) + '.ndjson');
    if (useCache && fs.existsSync(initiativesCache)) {
        const input = fs.createReadStream(initiativesCache);
        try {
            for (var _b = tslib_1.__asyncValues(readLines(input)), _c; _c = await _b.next(), !_c.done;) {
                const line = _c.value;
                issues.push(JSON.parse(line));
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    else {
        const issuesJira = await searchIssues_1.default(userConfig.jira, userConfig.roadmap.jqlInitiatives, 'summary,status,' +
            userConfig.jira.fields.points +
            ',' +
            userConfig.jira.fields.originalPoints +
            ',issuetype,assignee');
        const issueFileStream = fs.createWriteStream(initiativesCache, {
            flags: 'w'
        });
        for (let issue of issuesJira) {
            const updatedIssue = Object.assign(Object.assign({}, issue), { host: userConfig.jira.host, jql: userConfig.roadmap.jqlInitiatives, team: teamUtils_1.getTeamFromAssignee(issue, userConfig.roadmap.teams), points: returnTicketsPoints(issue, userConfig) });
            issueFileStream.write(JSON.stringify(jiraUtils_1.cleanIssue(updatedIssue)) + '\n');
            issues.push(jiraUtils_1.cleanIssue(updatedIssue));
        }
        issueFileStream.end();
    }
    cli_ux_1.default.action.stop(' done');
    return issues;
};
exports.default = fetchInitiatives;
const returnTicketsPoints = (issue, config) => {
    if (issue.fields[config.jira.fields.points] !== undefined &&
        issue.fields[config.jira.fields.points] !== null) {
        return issue.fields[config.jira.fields.points];
    }
    if (issue.fields[config.jira.fields.originalPoints] !== undefined &&
        issue.fields[config.jira.fields.originalPoints] !== null) {
        return issue.fields[config.jira.fields.originalPoints];
    }
    return 0;
};
//https://medium.com/@wietsevenema/node-js-using-for-await-to-read-lines-from-a-file-ead1f4dd8c6f
const readLines = (input) => {
    const output = new stream.PassThrough({ objectMode: true });
    const rl = readline.createInterface({ input });
    rl.on('line', line => {
        output.write(line);
    });
    rl.on('close', () => {
        output.push(null);
    });
    return output;
};
